@@ -226,9 +226,9 @@ https://my.org/q?x=1&y=2
         self.assertGreaterEqual(minor, 0)
         self.assertGreaterEqual(micro, 0)
 
-    def test_timezone_constructor_error(self):
-        self.assertRaisesRegexp(TypeError, '.*constructor.*help\(GLib.TimeZone\).*',
-                                GLib.TimeZone)
+    def test_timezone_constructor(self):
+        timezone = GLib.TimeZone("+05:21")
+        self.assertEqual(timezone.get_offset(0), ((5 * 60) + 21) * 60)
 
     def test_source_attach_implicit_context(self):
         context = GLib.MainContext.default()
